function varargout = mkSpectrum(varargin)
%                        SPECTRAL BINNING TOOL
%                    Michael Kelzenberg, Jan 2009
%                           mk@caltech.edu
%                 California Institute of Technology
%
% This is a GUI-based MATLAB tool for generating discrete approximations 
% of the continuous solar power spectrum.  The intended application is to
% approximate solar illumination as a combination of a finite number of
% single-wavelength excitation sources.
%  
% HOW TO INVOKE: 
%   At the matlab prompt, enter:
%     mkSpectrum
%   The mkSpectrum.fig and mkSpectrum.m files must lie within the current
%   directory.
%  
% INPUT FILES:
%   The program expects a text input file of spectral data.
%   Column 1:  Wavelength (nm)
%   Column 2:  Spectral power density (mW/cm2/nm)
%  
%   I have provided two such spectra with this program:
%     AM15G.asc
%     AM15D.asc
%   Which originate from the ASTM G173 spectra
%     http://rredc.nrel.gov/solar/spectra/am1.5/
% 
%   Note: The input data needn't be spaced equally in wavelength.
% 
% OUTPUT:
%   Upon successful processing, the program will output data as:
%    File - will be written to disk and opened in an editor (BinData.asc)
%    Workspace - data will be placed in MATLAB workspace (binData)
%     Col 1: Wavelength (nm)
%     Col 2: Power (mW/cm2)
%     Col 3: Current (mA/cm2)
% 
% PLOTTING:
%   The PLOT function generates a MATLAB plot of the specified spectral
%   region and displays the power and current within this region.
%   
% BINNING:
%   The BIN function generates an discrete list of single-wavelength
%   illumination sources based on a user-specified bin boundaries and
%   wavelengths.  The photons within each bin of the input spectrum are
%   summed and represented as a single "beam" of photons at a certain
%   wavelength.  The boundaries and beam wavelengths are calculated as
%   follows:
% 
%   BIN BOUNDARY METHODS:
%     From ... To  ... by: Equally-spaced bins in wavelength
%     User Specified:  A list of numbers (wavelengths) or any valid
%        MATLAB expressions may be entered here, for example:
%        300 400 450 500 550 600 700
%        [300 400:50:600 700]
%        1240./[3 2 1.12]       (useful conversion from eV)
%        myData                 (where myData is a workspace variable) 
%   
%   BEAM WAVELENGTH METHODS:
%     Midpoint:  The arithmetic mean of the bin boundaries
%     Conserve Energy:  This method examines the total energy, in
%       addition to the total number of photons, within each bin.  The
%       total energy is then divided by the number of photons to 
%       determine the photon energy (wavelength) of the resulting beam.
%     User Specified:  Similar syntax to user specified bin boundary 
%       method.  The size of this list must be one element shorter than
%       the number of boundaries.
%       NOTE: This method does not enforce that the beam energy lie 
%         within the bin it represents.  However it will print an warning
%         message to the screen.
% 
% IMPLEMENTATION:
%   The spectral conversion uses linear interpolation between data points
%   of the input spectrum, and oversampling numerical integration to
%   retain accuracy even with very small bins, or sparse input spectral
%   data.  Thus, it can be accurately used to re- or over-sample the 
%   input spectrum.  The accuracy can be confirmed by examining the
%   output file, which shows the pre- and post- binning values of total
%   power and current. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% MKSPECTRUM M-file for mkSpectrum.fig
%      MKSPECTRUM, by itself, creates a new MKSPECTRUM or raises the existing
%      singleton*.
%
%      H = MKSPECTRUM returns the handle to a new MKSPECTRUM or the handle to
%      the existing singleton*.
%
%      MKSPECTRUM('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in MKSPECTRUM.M with the given input arguments.
%
%      MKSPECTRUM('Property','Value',...) creates a new MKSPECTRUM or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before mkSpectrum_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to mkSpectrum_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help mkSpectrum

% Last Modified by GUIDE v2.5 17-Jan-2009 17:33:13

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @mkSpectrum_OpeningFcn, ...
                   'gui_OutputFcn',  @mkSpectrum_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before mkSpectrum is made visible.
function mkSpectrum_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to mkSpectrum (see VARARGIN)

% Choose default command line output for mkSpectrum
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);
disp('mkSpectrum');

% UIWAIT makes mkSpectrum wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = mkSpectrum_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in loadFileButton.
function loadFileButton_Callback(hObject, eventdata, handles)
% hObject    handle to loadFileButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global FileName;
FileName = loadSpectrum;

if( isempty(FileName) )
    return
end

global SPECDATA;
wls = SPECDATA(:,1);

global maxwl minwl;
minwl = wls(1);
maxwl = wls(end);

infostring = { ['Loaded ' FileName ': ' ...
    num2str(length(wls)) ' data points from ' num2str(minwl) 'nm to ' ...
    num2str(maxwl) ' nm. '] ...
    ['Power: ' num2str(binpwr(minwl, maxwl)) ' mW/cm2 '] ...
    ['Current: ' num2str(binphots(minwl, maxwl)*qe*1000) ' mA/cm2' ] };

set(findobj(gcf, 'Tag', 'fileStatusText'), 'String', infostring);


set(findobj(gcf, 'Tag', 'rangeFrom'), 'String', num2str(minwl) );
set(findobj(gcf, 'Tag', 'rangeTo'), 'String', num2str(maxwl) );
set(findobj(gcf, 'Tag', 'plotRangeButton'),'Enable','on');
set(findobj(gcf, 'Tag', 'rangeInfo'), 'String','' );
set(findobj(gcf, 'Tag', 'binInfo'), 'String','' );


function rangeFrom_Callback(hObject, eventdata, handles)
% hObject    handle to rangeFrom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of rangeFrom as text
%    str2double(get(hObject,'String')) returns contents of rangeFrom as a double

global minwl;
try
    rangeFrom = evalin('base', get(findobj(gcf,'Tag','rangeFrom'),'String'));
catch ME
    rangeFrom = minwl;
end
rangeTo = str2num(get(handles.rangeTo,'String'));

if (rangeFrom < minwl)
    rangeFrom = minwl;
end

if (rangeFrom > rangeTo)
    rangeFrom = minwl;
end

set(findobj(gcf,'Tag','rangeFrom'),'String', num2str(rangeFrom));

% --- Executes during object creation, after setting all properties.
function rangeFrom_CreateFcn(hObject, eventdata, handles)
% hObject    handle to rangeFrom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), ...
        get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function rangeTo_Callback(hObject, eventdata, handles)
% hObject    handle to rangeTo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of rangeTo as text
%      str2double(get(hObject,'String')) returns contents of rangeTo as a double
global maxwl;
try
    rangeTo = evalin('base',get(findobj(gcf,'Tag','rangeTo'),'String'));
catch ME
    rangeTo = maxwl;
end
rangeFrom = str2num(get(handles.rangeFrom,'String'));
if (rangeTo > maxwl || rangeTo < rangeFrom)
    rangeTo = maxwl;
end
set(findobj(gcf,'Tag','rangeTo'),'String', num2str(rangeTo));

% --- Executes during object creation, after setting all properties.
function rangeTo_CreateFcn(hObject, eventdata, handles)
% hObject    handle to rangeTo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), ...
        get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in plotRangeButton.
function plotRangeButton_Callback(hObject, eventdata, handles)
% hObject    handle to plotRangeButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
rangeFrom = eval(get(findobj(gcf,'Tag','rangeFrom'),'String'));
rangeTo = eval(get(findobj(gcf,'Tag','rangeTo'),'String'));

global SPECDATA;
wl_ = SPECDATA(:,1);
phot_ = SPECDATA(:,3);
pwr_ = SPECDATA(:,2);

wls = rangeFrom:((rangeTo-rangeFrom)/2999):rangeTo;
psds = wls;
csds = wls;
for n = 1:length(wls)
    psds(n) = pwrspectraldensity(wls(n));
    csds(n) = 1000*qe*photspectraldensity(wls(n));
end

totalcurrent = 1000*qe*binphots(rangeFrom, rangeTo);
totalpwr = binpwr(rangeFrom, rangeTo);


figure
subplot(2,1,1)
plot( wl_, pwr_, 'k' );
hold on;
area( wls, psds );
title( 'Power spectral density');
xlabel( 'Wavelength (nm)');
ylabel( 'mW/cm^2/nm' );

subplot(2,1,2)
plot( wl_, 1000*qe*phot_, 'k' );
hold on;
area( wls, csds );
title( 'Photocurrent spectral density');
xlabel( 'Wavelength (nm)');
ylabel( 'mA/cm^2/nm' );


set(handles.rangeInfo, 'String', [ num2str(rangeFrom) ' nm to ' ...
    num2str(rangeTo) ' nm: ' num2str(totalpwr) ' mW/cm2, ' ...
    num2str(totalcurrent) ' mA/cm2.' ] );




function binBoundaries_Callback(hObject, eventdata, handles)
% hObject    handle to binBoundaries (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
checkBins(handles);

% --- Executes during object creation, after setting all properties.
function binBoundaries_CreateFcn(hObject, eventdata, handles)
% hObject    handle to binBoundaries (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), ...
        get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function checkBins(handles)
    set(handles.binButton,'Enable','off');
    equalSpacing = get(handles.equalSpacing, 'Value');
    if (equalSpacing)
        ok = checkEqualSpacing(handles) ;
    else
        ok = checkBinBoundaries(handles) ;
    end
    if (~ok)
        return
    end
    specifyWavelengths = get(handles.specifyBinWavelengths, 'Value');
    if (specifyWavelengths)
        ok = checkBinWavelengths(handles);
    end
    if (~ok)
        return
    end
    set(handles.binButton,'Enable','on');
    set(handles.binInfo,'String','');


function ok = checkBinWavelengths(handles)
ok = 0;
try
  binWavelengths = unique(evalin('base', get(handles.binWavelengths,'String')));      
catch ME
  binWavelengths = unique(str2num(get(handles.binWavelengths,'String')));
end
if (length(binWavelengths) > 0)
        ok = 1;
        set(handles.binWavelengths, 'String', num2str(binWavelengths)); 
else
        set(handles.binInfo,'String','Error with bin wavelengths');
end


    
        
   
    
function ok = checkEqualSpacing(handles)
ok = 0;
try
    spacingFrom = evalin('base', get(handles.spacingFrom,'String'));
    spacingTo = evalin('base', get(handles.spacingTo,'String'));
    spacingBy = evalin('base', get(handles.spacingBy,'String'));
    if (spacingFrom < spacingTo && length(spacingFrom:spacingBy:spacingTo) > 0 )
        ok = 1;
    end
    set(handles.spacingFrom, 'String', num2str(spacingFrom));
    set(handles.spacingTo, 'String', num2str(spacingTo));
    set(handles.spacingBy, 'String', num2str(spacingBy));
        
catch ME
    set(handles.binInfo,'String','Error with bin spacing parameters');
end

function ok = checkBinBoundaries(handles)
ok = 0;
try
    binBoundaries = unique(evalin('base', get(handles.binBoundaries,'String')));
catch ME
    binBoundaries = unique(str2num(get(handles.binBoundaries,'String')));
end
    if (length(binBoundaries) > 1)
        ok = 1;
        set(handles.binBoundaries, 'String', num2str(binBoundaries));
    else
        set(handles.binInfo,'String','Error with bin boundaries');
    end



function spacingFrom_Callback(hObject, eventdata, handles)
% hObject    handle to spacingFrom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

checkBins(handles)

% --- Executes during object creation, after setting all properties.
function spacingFrom_CreateFcn(hObject, eventdata, handles)
% hObject    handle to spacingFrom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), ...
        get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function spacingTo_Callback(hObject, eventdata, handles)
% hObject    handle to spacingTo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

checkBins(handles)

% --- Executes during object creation, after setting all properties.
function spacingTo_CreateFcn(hObject, eventdata, handles)
% hObject    handle to spacingTo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), ...
    get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function spacingBy_Callback(hObject, eventdata, handles)
% hObject    handle to spacingBy (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

checkBins(handles)

% --- Executes during object creation, after setting all properties.
function spacingBy_CreateFcn(hObject, eventdata, handles)
% hObject    handle to spacingBy (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), ...
        get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in binButton.
function binButton_Callback(hObject, eventdata, handles)
% hObject    handle to binButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    equalSpacing = get(handles.equalSpacing, 'Value');
    if (equalSpacing)
        spacingFrom = evalin('base', get(handles.spacingFrom,'String'));
        spacingTo = evalin('base', get(handles.spacingTo,'String'));
        spacingBy = evalin('base', get(handles.spacingBy,'String'));
        bins = spacingFrom:spacingBy:spacingTo;
        if (bins(end) < spacingTo)
            bins(length(bins)+1) = spacingTo;
        end
    else
        bins = str2num(get(handles.binBoundaries,'String'));
    end
    
    conserveEnergy = get(handles.conserveEnergy, 'Value');
    if (conserveEnergy)
        centerpts = [];
    end
    midpointWavelength = get(handles.midpointWavelength, 'Value');
    if (midpointWavelength)
        centerpts = [];
        for n=1:(length(bins)-1)
            centerpts(n) = (bins(n) + bins(n+1))/2;
        end
    end
    specifyWavelengths = get(handles.specifyBinWavelengths, 'Value');
    if (specifyWavelengths)
        centerpts = str2num(get(handles.binWavelengths,'String'));
        if ( length(bins) ~= length(centerpts)+1 )
            errordlg(['Error with bin wavelengths specified.  For n bin '...
                'wavelengths, there should be n+1 bin boundaries specified.']);
            return
        end
    end
    
[totalinputcurrent totalinputpower centerpts beampwr specpwr beamphot ...
    warningstring] = spectralCalculator(bins, centerpts);

figure;
subplot(3,1,1)
plot(bins(1):0.1:bins(end), pwrspectraldensity(bins(1):0.1:bins(end)), 'k' );
hold on
plot(centerpts, specpwr, 'or');
title('Power spectral density');
xlabel('\lambda (nm)');
ylabel('mW cm^-^2 nm^-^1');
subplot(3,1,2)
stem(centerpts, beampwr);
title('Bin power');
xlabel('\lambda (nm)');
ylabel('mW cm^-^2');
subplot(3,1,3)
stem(centerpts, qe*1000*beamphot);
title('Bin photocurrent');
xlabel('\lambda (nm)');
ylabel('mA cm^-^2');
set( handles.binInfo, 'String', warningstring);
if (~isempty(warningstring))
    warndlg(warningstring);
end

global FileName
of = fopen('binData.asc', 'w');
fprintf(of,['Input File: ' FileName '\n' ...
  'Range of binning: ' num2str(bins(1)) ' nm to ' num2str(bins(end)) ' nm.\n'...
  'Power in this range: ' num2str(totalinputpower) ' mW/cm2 original, ' ...
  num2str(sum(beampwr)) ' mW/cm2 as binned\n' 'Current in this range: ' ...
  num2str(totalinputcurrent) ' mA/cm2 original, ' ...
  num2str(1000*qe*sum(beamphot)) ' mW/cm2 as binned\n\n' ...
  'WL (nm)   \tPWR (mW/cm2)\tCUR (mA/cm2)\tBINFROM (nm)\t BINTO (nm)']);
for n=1:length(centerpts)
    fprintf(of, '\n%10g\t%12g\t%12g\t%12g\t%10g', ...
        centerpts(n),beampwr(n),qe*1000*beamphot(n),bins(n),bins(n+1));
end
fclose(of);

assignin('base','binData',[ centerpts' beampwr' qe*1000*beamphot']);
edit('binData.asc');

function binWavelengths_Callback(hObject, eventdata, handles)
% hObject    handle to binWavelengths (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

checkBins(handles)

% --- Executes during object creation, after setting all properties.
function binWavelengths_CreateFcn(hObject, eventdata, handles)
% hObject    handle to binWavelengths (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), ...
        get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function psd = photspectraldensity(wl)
global wl_ phot_ maxwl minwl;
psd = interp1(wl_, phot_, wl);
%psd = nansum([psd;zeros(length(psd))]);


function psd = pwrspectraldensity(wl)
global wl_ pwr_ maxwl minwl;
psd = interp1(wl_, pwr_, wl);
%psd = nansum([psd;zeros(length(psd))]);



function bp = binphots(wl1, wl2)
bp = quadgk( @photspectraldensity, wl1, wl2, 'MaxIntervalCount', 5000);

function bp = binpwr(wl1, wl2) 
bp = quadgk( @pwrspectraldensity, wl1, wl2, 'MaxIntervalCount', 5000);


function FileName = loadSpectrum
%load raw spectum data from the ASTM G173-03 file
%col 1: Wavelength nm
%col 2: W*m-2*nm-1

global SPECDATA;

[FileName,PathName] = uigetfile('*.asc','Select spectrum data file');

SPECDATA = load([PathName FileName]);

%convert to mW/cm2/nm
SPECDATA(:,2) = SPECDATA(:,2)*1000/100/100;

%add col. for photons/cm2/nm
energies = 1.239841875 ./ (SPECDATA(:,1)./1000) * qe; %energy per photon in J

SPECDATA(:,3) = SPECDATA(:,2)./1000./energies;

%add col. for photocurrent/cm2/nm
SPECDATA(:,4) = SPECDATA(:,3).*qe;

global wl_ phot_ pwr_;

wl_ = [0; SPECDATA(:,1); 1E6];
pwr_ = [0; SPECDATA(:,2); 0];
%wl_ = [0 SPECDATA(:,1) 1E6];
phot_ = [0; SPECDATA(:,3); 0];



% --- Executes on button press in specifyBinWavelengths.
function specifyBinWavelengths_Callback(hObject, eventdata, handles)
% hObject    handle to specifyBinWavelengths (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of specifyBinWavelengths
checkBins(handles);


% --- Executes on button press in conserveEnergy.
function conserveEnergy_Callback(hObject, eventdata, handles)
% hObject    handle to conserveEnergy (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of conserveEnergy
checkBins(handles);


% --- Executes on button press in midpointWavelength.
function midpointWavelength_Callback(hObject, eventdata, handles)
% hObject    handle to midpointWavelength (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of midpointWavelength
checkBins(handles);


% --- Executes on button press in specifySpacing.
function specifySpacing_Callback(hObject, eventdata, handles)
% hObject    handle to specifySpacing (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of specifySpacing
checkBins(handles);


% --- Executes on button press in equalSpacing.
function equalSpacing_Callback(hObject, eventdata, handles)
% hObject    handle to equalSpacing (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of equalSpacing
checkBins(handles);

function [totalinputcurrent totalinputpower centerpts beampwr specpwr ...
    beamphot warnstring] = spectralCalculator(bins, centerpts)
global SPECDATA;

wl_ = SPECDATA(:,1);
phot_ = SPECDATA(:,3);
pwr_ = SPECDATA(:,2);

if (isempty(centerpts))
    autocenter = 1;
    centerpts = zeros(1,length(bins)-1);
else
    autocenter = 0;
end

numBins = length(centerpts);

beampwr = zeros(1,numBins);
specpwr = zeros(1,numBins);
beamphot = zeros(1,numBins);

totalInputCurrent = 1e3*qe*binphots(bins(1), bins(end)); %in mA/cm2
totalInputPower = quad( @pwrspectraldensity, bins(1), bins(end)); %in mW/cm2

warnstring = [];

for nBin=1:numBins
    
    photsInBin = binphots( bins(nBin), bins(nBin+1) );  %in num/cm2/sec
    
    energyInBin = binpwr( bins(nBin), bins(nBin+1) ) / 1000 / qe; %in eV/cm2/sec
    
    if (autocenter)
        photEnergy = energyInBin / photsInBin;
        photWl = 1.239841875 / photEnergy * 1000;
        centerpts(nBin) = photWl;
    else
        photEnergy = 1.239841875 / (centerpts(nBin) / 1000);
    end
    beampwr(nBin) = photsInBin * photEnergy * qe * 1000; % now in mW
    specpwr(nBin) = beampwr(nBin) / (bins(nBin+1) - bins(nBin));
    beamphot(nBin) = photsInBin;
    
    if ( centerpts(nBin) > bins(nBin+1) || centerpts(nBin) < bins(nBin) )
        warnstring = ['Warning: At least one bin wavelength lies outside ' ...
            'the boundaries for that bin.'];
    end
        
end

totalinputcurrent = totalInputCurrent;
totalinputpower = totalInputPower;

function x = qe
x = 1.60217646E-19;


% --- Executes on button press in readmeButton.
function readmeButton_Callback(hObject, eventdata, handles)
% hObject    handle to readmeButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
edit('README.txt');
